﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Analysis;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("CurrentXCSSolutionAnalyzer", "")]
  [StorableClass]
  public sealed class CurrentXCSSolutionAnalyzer : XCSSolutionAnalyzer {
    private const string CurrentSolutionName = "Current Solution";
    private const string TrainingAccuracyDataTableName = "TrainingAccuracy";

    [StorableConstructor]
    private CurrentXCSSolutionAnalyzer(bool deserializing) : base(deserializing) { }
    private CurrentXCSSolutionAnalyzer(CurrentXCSSolutionAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new CurrentXCSSolutionAnalyzer(this, cloner);
    }
    public CurrentXCSSolutionAnalyzer()
      : base() {
    }

    protected override void UseCurrentXCSSolution(IXCSSolution xcsSolution) {
      XCSSolution xcs;
      if (!Results.ContainsKey(CurrentSolutionName)) {
        Results.Add(new Result(CurrentSolutionName, (IXCSSolution)xcsSolution.Clone()));
        xcs = Results[CurrentSolutionName].Value as XCSSolution;
        DataTable dataTable = new DataTable("Training Accuracy DataTable");
        xcs.Add(new Result(TrainingAccuracyDataTableName, dataTable));
        DataRow row = new DataRow("TrainingAccuracy");
        dataTable.Rows.Add(row);
        row.Values.Add(xcs.TrainingAccuracy);
      } else {
        DataTable dataTable = (Results[CurrentSolutionName].Value as XCSSolution)[TrainingAccuracyDataTableName].Value as DataTable;
        Results[CurrentSolutionName].Value = (IXCSSolution)xcsSolution.Clone();
        xcs = Results[CurrentSolutionName].Value as XCSSolution;
        dataTable.Rows["TrainingAccuracy"].Values.Add(xcs.TrainingAccuracy);
        xcs.Add(new Result(TrainingAccuracyDataTableName, dataTable));
      }
    }
  }
}
